/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_WEIGHT_MATRIX_SEARCH_DIALOG_H_
#define _GB2_WEIGHT_MATRIX_SEARCH_DIALOG_H_

#include <core_api/core_api.h>
#include <core_api/LRegion.h>

#include <ui/ui_PWMSearchDialog.h>

#include <QtCore/QList>
#include <QtGui/QTreeWidgetItem>
#include <QtGui/QCloseEvent>
#include <QtCore/QTimer>

namespace GB2 {

class ADVSequenceObjectContext;
class DNASequenceObject;
class WeightMatrixSearchTask;
class Task;
class WeightMatrixResultItem;
class PFMatrix;
class PWMatrix;

class PWMSearchDialogController : public QDialog, public Ui_PWMSearchDialog {
    Q_OBJECT
public:
    PWMSearchDialogController(ADVSequenceObjectContext* ctx, QWidget *p = NULL);
    ~PWMSearchDialogController();

public slots:
    virtual void reject();

protected:
    bool eventFilter(QObject *obj, QEvent *ev);

private slots:
    
    //buttons:
    void sl_selectModelFile();
    void sl_onSaveAnnotations();
    void sl_onClearList();
    void sl_onSearch();
    void sl_onClose();
    void sl_onBuildMatrix();
    void sl_onSearchJaspar();

    void sl_onSliderMoved(int value);
    void sl_onAlgoChanged(QString newAlgo);

    // groups
    void sl_onTaskFinished();
    void sl_onTimer();

    void sl_onResultActivated(QTreeWidgetItem* i, int col);
    
private:
    void connectGUI();
    void updateState();
    void updateStatus();
    void updateModel(const PWMatrix& m);
    
    bool checkPrevSettings();
    void savePrevSettings();

    void runTask();

    void importResults();

private:
    ADVSequenceObjectContext*   ctx;
    LRegion                     initialSelection;
    
    PWMatrix*       model;

    WeightMatrixSearchTask*     task;
    QTimer* timer;
};

}//namespace

#endif
