/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "BowtieConstants.h"
#include "BowtieAdapter.h"
#include "BowtieIOAdapter.h"

#include <core_api/DNAAlphabet.h>
#include <core_api/Task.h>
#include <core_api/Log.h>
#include <datatype/MAlignment.h>
#include <datatype/DNASequence.h>

#include <QtCore/QVector>

#include <stdlib.h>
#include <stdexcept>
#include <vector>

using namespace std;
using namespace seqan;

extern void resetSearchOptions();

void driverAdapter(const QList<GB2::DNASequence>& reads,
				   GB2::MAlignment& resultMA,
				   const char * type,
				   const string& ebwtFileBase,
				   const string& query,
				   const vector<string>& queries,
				   const vector<string>& qualities,
				   const string& outfile);

int bowtieBuildAdapter(const string& infile, const string& outfile);

namespace GB2 {

void BowtieAdapter::doBowtie(const QString& ebwtFileName, const QList<GB2::DNASequence> &reads, MAlignment& output, TaskStateInfo& ti) {
	try {
		resetSearchOptions();
		//driver<String<Dna, Alloc<> > >("DNA", ebwtFileName, query, queries, qualities, outfile);
		vector<string> queries;
		queries.push_back("reads/reads");
		driverAdapter(reads, output, "DNA", ebwtFileName.toAscii().constData(), "", queries, vector<string>(), "");
		if(output.getRows().count() == 0) {
			ti.setError(tr("Reference assembly failed - no possible alignment found"));
			return;
		}
		output.setAlphabet(reads.first().alphabet);
		//CHUD_STOP();
	}catch(exception& e) {
		cerr << "Exeption: " << e.what() << endl;
		ti.setError(QString("Bowtie exits with error \"%1\"").arg(e.what()));
	} catch(int e) {
		if(e != 0) {
			cerr << "Exeption: " << e << endl;
			ti.setError(QString("Bowtie exits with error code %1").arg(e));
		}
	}
}



void BowtieAdapter::doBowtieBuild(const QString& infile, const QString& outfile, TaskStateInfo& ti )
{
	try{
		bowtieBuildAdapter(infile.toLocal8Bit().constData(), outfile.toLocal8Bit().constData());
	} catch(std::exception& e) {
		ti.setError(QString("Bowtie-build exits with error: %1").arg(e.what()));
	} catch(int e) {
		ti.setError(QString("Bowtie-build exits with error code %1").arg(e));
	}
}

}