/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "BowtieConstants.h"
#include "BowtiePlugin.h"
#include "BowtieTask.h"

#include <core_api/AppContext.h>
#include <core_api/Task.h>
#include <core_api/Log.h>
#include <core_api/DocumentModel.h>
#include <core_api/DnaAssemblyAlgRegistry.h>
#include <util_ov_msaedit/MSAEditor.h>
#include <util_ov_msaedit/MSAEditorFactory.h>
#include <gobjects/MAlignmentObject.h>
#include <gobjects/GObjectTypes.h>

#include <util_algorithm/GAutoDeleteList.h>
#include <util_gui/GUIUtils.h>
#include <util_gui/DialogUtils.h>

#include <test_framework/xmltest/XMLTestFormat.h>
#include <test_framework/GTest.h>
#include <test_framework/GTestFrameworkComponents.h>
#include <test_framework/xmltest/DocumentModelTests.h>

#include "bowtie_tests/bowtieTests.h"

#include <QtGui/QDialog>
#include <QtGui/QFileDialog>

#define BOWTIE_ALG_NAME "Bowtie"

namespace GB2 {

extern "C" Q_DECL_EXPORT Plugin* GB2_PLUGIN_INIT_FUNC() {
    BowtiePlugin * plug = new BowtiePlugin();
    return plug;
}

LogCategory log(ULOG_CAT_BOWTIE);

    
BowtiePlugin::BowtiePlugin() 
: Plugin(tr("Bowtie"), 
		 tr("An ultrafast memory-efficient short read aligner, http://bowtie-bio.sourceforge.net")),
         ctx(NULL)
{
	GTestFormatRegistry* tfr = AppContext::getTestFramework()->getTestFormatRegistry();
    XMLTestFormat *xmlTestFormat = qobject_cast<XMLTestFormat*>(tfr->findFormat("XML"));
    assert(xmlTestFormat!=NULL);

    GAutoDeleteList<XMLTestFactory>* l = new GAutoDeleteList<XMLTestFactory>(this);
    l->qlist = BowtieTests ::createTestFactories();

    foreach(XMLTestFactory* f, l->qlist) { 
        bool res = xmlTestFormat->registerTestFactory(f);
        Q_UNUSED(res);
        assert(res);
    }

 	DnaAssemblyAlgRegistry* reg = AppContext::getDnaAssemblyAlgRegistry();
	DnaAssemblyToRefTaskFactory* alg = new BowtieTask::Factory();
 	reg->registerTask(alg, BOWTIE_ALG_NAME);
}

BowtiePlugin::~BowtiePlugin() {
    //nothing to do
}

}//namespace

