/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "BowtieConstants.h"
#include "BowtieTask.h"
#include "BowtieAdapter.h"
#include "BowtieContext.h"

#include <core_api/AppContext.h>
#include <core_api/AppSettings.h>
#include <core_api/AppResources.h>
#include <core_api/Log.h>
#include <core_api/StateLockableDataModel.h>
#include <core_api/DocumentModel.h>
#include <core_api/DocumentFormats.h>
#include <core_api/IOAdapter.h>
#include <core_api/Counter.h>
#include <core_api/ProjectModel.h>
#include <gobjects/DNASequenceObject.h>
#include <util_tasks/LoadDocumentTask.h>
#include <document_format/DocumentFormatUtils.h>

#include <QtCore/QMutexLocker>

namespace GB2 {

static LogCategory log(ULOG_CAT_BOWTIE);

QMutex BowtieBuildTask::mutex;

BowtieTask::BowtieTask(const DnaAssemblyToRefTaskSettings & _config) : DnaAssemblyToReferenceTask(_config, TaskFlags_NR_FOSCOE)
{
    GCOUNTER( cvar, tvar, "BowtieTask" );
	tlsTask = NULL;
	buildTask = NULL;
	//resultMA.setAlphabet(inputMA.getAlphabet());
    //config.nThreads = (config.nThreads == 0) ? AppContext::getAppSettings()->getAppResourcePool()->getIdealThreadCount():config.nThreads;
    //assert(config.nThreads > 0);
    setMaxParallelSubtasks(1);
    //todo: make more precise estimation, use config.op mode
    //int aliLen = ma.getLength();
    //int nSeq = ma.getNumRows();
    //int memUseMB = int(quint64(200 * aliLen * nSeq) / (1024 * 1024)); //200x per char in alignment
    //TaskResourceUsage tru(RESOURCE_MEMORY, memUseMB);
    
//    taskResources.append(tru);
	QString indexURL(settings.refSeqUrl.getURLString());
	if(settings.usePrebuiltIndex) {
		QRegExp rx("(.+)(\\.rev)?\\.\\d\\.ebwt");
		if(rx.indexIn(indexURL) != -1) {
			indexPath = rx.cap(1);
		} else {
			stateInfo.setError("Invalid ebwt index path");
			return;
		}
	} else {
		//Build index
		buildTask = new BowtieBuildTask(indexURL, settings.resultFileName.getURLString()); 
		buildTask->setSubtaskProgressWeight(0.6);
		addSubTask(buildTask);
	}
	tlsTask = new BowtieTLSTask();
	tlsTask->setSubtaskProgressWeight(0.4);
	addSubTask(tlsTask);
}

QList<Task*> BowtieTask::onSubTaskFinished(Task* subTask) {
	Q_UNUSED(subTask);
	QList<Task*> res;
	if(subTask->hasErrors()) {
		return res;
	}
	if(subTask == buildTask)
		indexPath = static_cast<BowtieBuildTask*>(subTask)->getEbwtPath();
	return res;
}

Task::ReportResult BowtieTask::report() {
    return ReportResult_Finished;
}

/************************************************************************/
/* BowtieTLSTask                                                        */
/************************************************************************/

BowtieTLSTask::BowtieTLSTask() : TLSTask("Bowtie TLS Task", TaskFlags_FOSCOE)
{
	tpm = Task::Progress_Manual;
}

void BowtieTLSTask::_run()
{
	BowtieTask* parent = static_cast<BowtieTask*>(getParentTask());
	BowtieAdapter::doBowtie(parent->indexPath, parent->settings.shortReads, parent->result, stateInfo);
}

TLSContext* BowtieTLSTask::createContextInstance()
{
	return new BowtieContext(stateInfo);
}
/************************************************************************/
/* BowtieBuildTask                                                      */
/************************************************************************/

BowtieBuildTask::BowtieBuildTask( QString _refPath, QString _outEbwtPath )
:TLSTask(tr("Bowtie Build")), refPath(_refPath), outEbwtPath(_outEbwtPath)
{
	//do nothing	
}

TLSContext* BowtieBuildTask::createContextInstance() {
	return new BowtieContext(stateInfo);
}

void BowtieBuildTask::_run()
{
	QMutexLocker lock(&mutex);
	BowtieAdapter::doBowtieBuild(this->refPath, this->outEbwtPath, stateInfo);
}

Task::ReportResult BowtieBuildTask::report()
{
	//TODO:
	return ReportResult_Finished;
}

} //namespace
