/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_BOWTIE_TASK_H_
#define _GB2_BOWTIE_TASK_H_

#include <core_api/Task.h>
#include <util_tasks/TLSTask.h>
#include <util_dna_assembly/DnaAssemblyTask.h>
#include <gobjects/MAlignmentObject.h>

#include <QtCore/QPointer>
#include <QtCore/QMutex>

namespace GB2 {

class StateLock;
class MAlignmentObject;
class LoadDocumentTask;
class BowtieTLSTask;
class BowtieTask;
class BowtieBuildTask;

class BowtieTask : public DnaAssemblyToReferenceTask {
    Q_OBJECT
	DNA_ASSEMBLEY_TO_REF_TASK_FACTORY(BowtieTask)
	friend class BowtieTLSTask;
public:
    BowtieTask(const DnaAssemblyToRefTaskSettings & config);
	QList<Task*> onSubTaskFinished(Task* subTask);
    ReportResult report();
private:	
	BowtieTLSTask* tlsTask;
	BowtieBuildTask* buildTask;
	QString indexPath;
};

class BowtieTLSTask : public TLSTask {
	Q_OBJECT
public:
	BowtieTLSTask();	
protected:
	void _run();
	TLSContext* createContextInstance();
};

class BowtieBuildTask : public TLSTask {
	Q_OBJECT
public:
	BowtieBuildTask(QString refPath, QString outEbwtPath);
	
	ReportResult report();
	QString getEbwtPath() const { return outEbwtPath; }
protected:
	void _run();
	TLSContext* createContextInstance();
private:
	static QMutex mutex;
	QString refPath;
	QString outEbwtPath;
};

}//namespace
#endif
