/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "Muscle4Tests.h"
#include "Muscle4Task.h"

#include <core_api/DocumentModel.h>
#include <gobjects/MAlignmentObject.h>

namespace GB2 {

#define IN_FILE_NAME_ATTR "in"
#define DOC1_ATTR "doc1"
#define DOC2_ATTR "doc2"

struct GTestBoolProperty {
    static bool get(QString attr, bool &value, const QDomElement &el) {
        QString value_str = el.attribute(attr);
        if(!value_str.isEmpty()) {
            bool ok = false;
            value = value_str.toInt(&ok);
            return ok;
        }
        return true;
    }
};

#define GET_BOOL_PROP(ATTR,VAL) if(GTestBoolProperty::get((ATTR),(VAL),(el))==false) {\
    failMissingValue((ATTR));\
    return;}

void GTest_muscle4::init(XMLTestFormat *tf, const QDomElement& el) {
    Q_UNUSED(tf);

    ma_result = NULL;

    inputDocCtxName = el.attribute(IN_FILE_NAME_ATTR);
    if (inputDocCtxName.isEmpty()) {
        failMissingValue(IN_FILE_NAME_ATTR);
        return;
    }
}

void GTest_muscle4::prepare() {
    mTask = NULL;
    ma_result = NULL;

    doc = getContext<Document>(this, inputDocCtxName);
    if (doc == NULL) {
        stateInfo.setError(  QString("context not found %1").arg(inputDocCtxName) );
        return;
    }

    QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
    if (list.size() == 0) {
        stateInfo.setError(  QString("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
        return;
    }

    GObject *obj = list.first();
    if(obj==NULL){
        stateInfo.setError(  QString("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
        return;
    }
    assert(obj!=NULL);
    MAlignmentObject* ma = qobject_cast<MAlignmentObject*>(obj);
    if(ma==NULL){
        stateInfo.setError(  QString("error can't cast to multiple alignment from GObject") );
        return;
    }

    ma_result = ma;
    mTask = new Muscle4GObjectTask(ma_result);
    addSubTask(mTask);
}

Task::ReportResult GTest_muscle4::report() {
    if (!hasErrors()) {
        if (mTask->hasErrors()) {
            stateInfo.setError(  mTask->getError() );
            return ReportResult_Finished;
        }
    }
    return ReportResult_Finished;
}

QList<XMLTestFactory*> Muscle4Tests::createTestFactories() {
    QList<XMLTestFactory*> res;
    res.append(GTest_muscle4::createFactory());
    return res;
}

}//namespace
