/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>

#include <core_api/DnaAssemblyAlgRegistry.h>
#include <core_api/DocumentModel.h>
#include <core_api/DocumentFormats.h>
#include <core_api/DocumentUtils.h>
#include <core_api/GUrlUtils.h>
#include <util_gui/DialogUtils.h>

#include "DnaAssemblyDialog.h"


namespace GB2 {

DnaAssemblyDialog::DnaAssemblyDialog(const DnaAssemblyAlgRegistry* registry, QWidget* p /*= NULL*/ )
: QDialog(p), assemblyRegistry(registry)
{
    setupUi(this);
    QStringList names = registry->getTaskNameList();
    methodNamesBox->addItems(names);
    // TODO: change the way default method is set
    methodNamesBox->setCurrentIndex(1);
    sl_onAlgorithmChanged(methodNamesBox->currentText());
    connect(addShortreadsButton, SIGNAL(clicked()), SLOT(sl_onAddShortReadsButtonClicked()) );
    connect(removeShortReadsButton, SIGNAL(clicked()), SLOT(sl_onRemoveShortReadsButtonClicked()));
    connect(setResultFileNameButton, SIGNAL(clicked()), SLOT(sl_onSetResultFileNameButtonClicked()));
    connect(addRefButton, SIGNAL(clicked()), SLOT(sl_onAddRefButtonClicked()) );
	connect(methodNamesBox, SIGNAL(currentIndexChanged(const QString &)), SLOT(sl_onAlgorithmChanged(const QString &)));
}

void DnaAssemblyDialog::updateState()
{
    
}

void DnaAssemblyDialog::sl_onAddShortReadsButtonClicked() {
    LastOpenDirHelper lod;
    QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Add short reads"), lod.dir);
    if (fileNames.isEmpty()) {
        return;
    }
    lod.url = fileNames.at(fileNames.count() - 1);
    foreach(const QString& f, fileNames) {
        shortReadsList->addItem(f);    
    }
}

void DnaAssemblyDialog::sl_onAddRefButtonClicked() {
    LastOpenDirHelper lod;
	QString filter;
	/*if(prebuiltIndexBox->isChecked()) 
		filter = DialogUtils::prepareDocumentsFileFilter(BaseDocumentFormats::INDEX, true);
	else 
		filter = DialogUtils::prepareDocumentsFileFilter(BaseDocumentFormats::RAW_DNA_SEQUENCE, true);
    */

    lod.url = QFileDialog::getOpenFileName(this, tr("Open reference sequence"), lod.dir, filter);
    if (lod.url.isEmpty()) {
        return;
    }

    refSeqEdit->setText(lod.url);
    GUrl refUrl = lod.url;
    GUrl url = GUrlUtils::rollFileName(refUrl.dirPath() + "/" + refUrl.baseFileName()+ ".srfa", DocumentUtils::getNewDocFileNameExcludesHint());
    resultFileNameEdit->setText(url.getURLString());

}

void DnaAssemblyDialog::accept()
{
    if (refSeqEdit->text().isEmpty()) {
		if(prebuiltIndexBox->isChecked()) {
			QMessageBox::information(this, tr("DNA Assembly"),
				tr("Index url is not set!") );
		}
		else {
			QMessageBox::information(this, tr("DNA Assembly"),
				tr("Reference sequence url is not set!") );
		}
    } else if (resultFileNameEdit->text().isEmpty() ) {
        QMessageBox::information(this, tr("DNA Assembly"),
            tr("Result alignment file name is not set!") );
    } else if (shortReadsList->count() == 0 ) {
        QMessageBox::information(this, tr("DNA Assembly"),
            tr("Short reads list is empty!") );
    } else {   
        QDialog::accept();
    }
}

const GUrl DnaAssemblyDialog::getRefSeqUrl()
{
    return refSeqEdit->text();
}

const QList<GUrl> DnaAssemblyDialog::getShortReadUrls()
{
    QList<GUrl> urls;
    int numItems = shortReadsList->count();
    for( int i =0; i < numItems; ++i) {
        urls.append(shortReadsList->item(i)->text());
    }
    return urls;
}

const QString DnaAssemblyDialog::getAlgorithmName()
{
    return methodNamesBox->currentText();
}

bool DnaAssemblyDialog::getUsePrebuiltIndex()
{
	return prebuiltIndexBox->isChecked();
}

void DnaAssemblyDialog::sl_onRemoveShortReadsButtonClicked()
{
    int currentRow = shortReadsList->currentRow();
    shortReadsList->takeItem(currentRow);
    
}

void DnaAssemblyDialog::sl_onSetResultFileNameButtonClicked()
{
    LastOpenDirHelper lod;
    lod.url = QFileDialog::getSaveFileName(this, tr("Set result alignment file name"), lod.dir);
    if (!lod.url.isEmpty()) {
        GUrl result = lod.url;
        if (result.lastFileSuffix().isEmpty()) {
            result = QString( "%1.srfa" ).arg( result.getURLString() );
        }
        resultFileNameEdit->setText(result.getURLString());
    }
}


void DnaAssemblyDialog::sl_onAlgorithmChanged(const QString &text)
{
	//TODO: add custom settings, remove hard coding below
	prebuiltIndexBox->setEnabled(getAlgorithmName() == "Bowtie");
}

const QString DnaAssemblyDialog::getResultFileName()
{
    return resultFileNameEdit->text();
}

int DnaAssemblyDialog::getNMismatches()
{
    return mismatchesBox->value();
}

} // GB2

